#General 1st Person Camera Movement Class
#Virtual Environments 2007
#Palli og Palmi
#
from direct.showbase.DirectObject import DirectObject
from direct.gui.OnscreenText import OnscreenText
from pandac.PandaModules import *
import sys

key_map = (('s',['back',1]),('s-up',['back',0]),
('w',['forward',1]),('w-up',['forward',0]),
('a',['strafe_left',1]),('a-up',['strafe_left',0]),
('d',['strafe_right',1]),('d-up',['strafe_right',0]),
('arrow_left',['left',1]),('arrow_left-up',['left',0]),
('arrow_right',['right',1]),('arrow_right-up',['right',0]),
('arrow_up',['up',1]),('arrow_up-up',['up',0]),
('arrow_down',['down',1]),('arrow_down-up',['down',0]),
#shift keys
('shift-s',['shift_back',1]),('shift-s-up',['shift_back',0]),
('shift-w',['shift_forward',1]),('shift-w-up',['shift_forward',0]),
('shift-a',['shift_strafe_left',1]),('shift-a-up',['shift_strafe_left',0]),
('shift-d',['shift_strafe_right',1]),('shift-d-up',['shift_strafe_right',0]),
('shift-arrow_left',['shift_left',1]),('shift-arrow_left-up',['shift_left',0]),
('shift-arrow_right',['shift_right',1]),('shift-arrow_right-up',['shift_right',0]),
('shift-arrow_up',['shift_up',1]),('shift-arrow_up-up',['shift_up',0]),
('shift-arrow_down',['shift_down',1]),('shift-arrow_down-up',['shift_down',0]),
('shift', ['shift_isdown', 1]), ('shift-up', ['shift_isup', 0])
)


class Movement(DirectObject):
  def __init__(self, parent=None, mouseLook=True, showCoordinates=True):

    self.parent = parent
    self.showCoordinates = showCoordinates
    self.mouseLook = mouseLook
    # Set up movement keys
    self.keystate = {'back':0, 'forward':0, 'left':0, 'right':0,
                     'strafe_left':0, 'strafe_right':0, 'up':0, 'down':0,
                     'shift_back':0, 'shift_forward':0, 'shift_left':0, 'shift_right':0,
                     'shift_strafe_left':0, 'shift_strafe_right':0, 'shift_up':0, 'shift_down':0,
                     'shift_isdown':0, 'shift_isup':0} # Initial state of keys

    self.shift = 0
    for key in key_map:
      self.accept(key[0], self.set_keystate, key[1])


    if showCoordinates==True:
      self.textCoordinates = OnscreenText(pos = (0, -0.9), fg = (1,1,1,1), bg = (0.3, 0.3, 0.3, 0.5),
                                          align = TextNode.ACenter, scale = 0.07, mayChange = True)

    self.lasttime = 0
    taskMgr.add(self.move, "move") # Start another independent update loop


  def set_keystate(self, key, state):
    """ Set a stored state (0,1) for a specified key ('back',...)"""
    tempkey = key
    if key=='shift_isup':
      self.shift=0
    if key=='shift_isdown':
      self.shift=1

    tempkey = key.replace("shift_","")
    self.keystate[tempkey] = state

  def move(self, task):
    """
    Update camera position and/or rotation for all active key states.
    """
    elapsed = task.time - self.lasttime

    #get current camera a pitch and set it to 0 so camera will not fly or go through ground
    pitch = base.camera.getP()
    base.camera.setP(0)
    base.camera.setR(0)

    #constants
    movement_constant = 2
    look_constant = 50
    mouse_look_constant = 100
    run_constant = 1
    if self.shift is 1:
      run_constant=2

      #movement
    if self.keystate['back'] is 1:
      base.camera.setPos(camera,0,-movement_constant*run_constant*elapsed,0)
    if self.keystate['forward'] is 1:
      base.camera.setPos(camera, 0, movement_constant*run_constant*elapsed, 0)
    if self.keystate['strafe_left'] is 1:
      base.camera.setPos(camera,-movement_constant*run_constant*elapsed, 0,0)
    if self.keystate['strafe_right'] is 1:
      base.camera.setPos(camera,movement_constant*run_constant*elapsed, 0,0)

    #set the pitch again
    base.camera.setP(pitch)
    #looking keyboard
    if self.keystate['right'] is 1:
      base.camera.setH(camera.getH()-look_constant*elapsed)
    if self.keystate['left'] is 1:
      base.camera.setH(camera.getH()+look_constant*elapsed)
    if self.keystate['up'] is 1:
      tempP = camera.getP()+look_constant*elapsed
      if tempP >35:
        tempP=35
      base.camera.setP(tempP)
    if self.keystate['down'] is 1:
      tempP = camera.getP()-look_constant*elapsed
      if tempP < -35:
        tempP = -35
      base.camera.setP(tempP)

    #looking mouse
    if base.mouseWatcherNode.hasMouse() and self.mouseLook:
      md = base.win.getPointer(0)
      x = md.getX()
      y = md.getY()
      deltax =  (x - base.win.getXSize()/2)*0.1
      deltay =  (y - base.win.getYSize()/2)*0.1
      base.win.movePointer(0, base.win.getXSize()/2, base.win.getYSize()/2)
      if (deltax != None and deltax !=0):
        base.camera.setH(camera.getH()-deltax)
      if (deltay != None and deltay != 0):
        tempP = camera.getP()-deltay
        if tempP >35:
          tempP=35
        if tempP < -35:
          tempP = -35
        base.camera.setP(tempP)

    self.lasttime=task.time

    #set location text
    if self.showCoordinates==True:
      self.textCoordinates.setText("Your location is (%2.2f,%2.2f,%2.2f)" % (base.camera.getX(), base.camera.getY(), base.camera.getZ()))
    #call parent movement function if it wants to do anything when camera has moved
    if self.parent is not None:
      self.parent.movement()

    return task.cont
